﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

// installed with the RsScope 3.35.4 or newer
using RohdeSchwarz.RsScope;

// installed with Ivi shared components 2.42+ and Ivi.NET shared components 1.4.0+
// source: http://www.ivifoundation.org
using Ivi.Driver; 
using Ivi.Scope;

namespace Csharp_RsScope_Driver_Example
{
    class Program
    {
        static void Main(string[] args)
        {
            RsScope scope = null;
            try //separate try-catch for scope initialization prevents accessing uninitialized object
            {
                //-----------------------------------------------------------
                // Initialization:
                //-----------------------------------------------------------
                // Adjust the VISA Resource string to fit your instrument
                scope = new RsScope("TCPIP::192.168.1.101::INSTR", true, true, "Simulate=False");
                scope.UtilityFunctions.OPCTimeout = 3000; //Timeout for driver wait operation, e.g. waveform acquisition
            }
            catch (Ivi.Driver.IOException e)
            {
                Console.WriteLine("Error initializing the driver session:\n{0}", e.Message);
                Console.WriteLine("Press any key to finish.");
                Console.ReadKey();
                return;
            }

            try //# try block to catch any InstrumentStatusException
            {
                var idnResponse = scope.UtilityFunctions.IDQueryResponse;
                Console.WriteLine("Instrument Identification string:\n{0}", idnResponse);
                scope.Utility.Reset(); // Reset the instrument, clear the Error queue
                scope.Display.DisplayUpdate = true; // Display update ON - switch OFF after debugging
                //-----------------------------------------------------------
                // Basic Settings:
                //-----------------------------------------------------------
                scope.Acquisition.HorizontalRecordSelection = RecordSelection.RecordTime; // Define Horizontal scale by number of points
                scope.Acquisition.HorizontalTimePerRecord = Ivi.Driver.PrecisionTimeSpan.FromMilliseconds(10); // 10ms Acquisition time
                scope.Acquisition.HorizontalRecordLength = 20002; // 20002 X points
                scope.Channel["CH1"].Configure(2.000, 0.0, RohdeSchwarz.RsScope.VerticalCoupling.AC, true); // Channel vertical settings
                //-----------------------------------------------------------
                // Trigger Settings:
                //-----------------------------------------------------------
                scope.Trigger["TrigA"].Modifier = RohdeSchwarz.RsScope.TriggerModifier.Normal; // Trigger Auto mode in case of no signal is applied
                scope.Trigger["TrigA"].Source = RohdeSchwarz.RsScope.TriggerSource.Channel1; // Trigger source CH1
                scope.Trigger["TrigA"].Type = RohdeSchwarz.RsScope.TriggerType.Edge;
                scope.Trigger["TrigA"].Edge.Slope = RohdeSchwarz.RsScope.Slope.Positive;
                scope.Trigger["TrigA"].Channel["CH1"].Level = 0.04; //setting the trigger level on CH_1 to 40mV
                var opc = scope.UtilityFunctions.QueryOPC; //Using *OPC? query waits until all the instrument settings are finished
                // -----------------------------------------------------------
                // SyncPoint 'SettingsApplied' - all the settings were applied
                // -----------------------------------------------------------
                // Arming the SCOPE for single acquisition
                // -----------------------------------------------------------
                scope.UtilityFunctions.OPCTimeout = 3000; // Acquisition timeout - set it higher than the acquisition time
                scope.WaveformAcquisition.RunSingleWithoutWait();
                // -----------------------------------------------------------
                // DUT_Generate_Signal() - in our case we use Probe compensation signal
                // where the trigger event (positive edge) is reoccuring
                // -----------------------------------------------------------
                scope.WaveformAcquisition.WaitForMeasurementComplete(3000); // Acquisition timeout - set it higher than the acquisition time
                // -----------------------------------------------------------
                // SyncPoint 'AcquisitionFinished' - the results are ready
                // -----------------------------------------------------------
                // Fetching the waveform - only Binary format is available in drivers
                // -----------------------------------------------------------
                IWaveform<double> waveform = null;
                waveform = scope.Channel["CH1"].Waveform["W0"].FetchWaveform(waveform);
                Console.WriteLine("Instrument returned {0} samples in the waveform structure", waveform.ValidPointCount);
                // -----------------------------------------------------------
                // Making a instrument screenshot and transferring the file to the PC
                // -----------------------------------------------------------
                string instrScreenshotFile = "c:\\temp\\Dev_Screenshot.png";
                string PCscreenshotFile = "c:\\temp\\PC_Screenshot.png";
                scope.HardcopyAndPrinter.Destination["DE1"].DeviceDestination = HardcopyDevice.File; // Create Hardcopy as file
                scope.HardcopyAndPrinter.Destination["DE1"].DeviceLanguageOutputFormat = HardcopyDeviceLang.png; // Set the screenshot format
                scope.HardcopyAndPrinter.FileName = instrScreenshotFile;
                scope.HardcopyAndPrinter.Print(); // Make the screenshot now
                scope.DataManagement.ReadToFileFromInstrument(instrScreenshotFile, PCscreenshotFile); // Copy the file from Scope to the PC
                Console.WriteLine("Screenshot file saved to PC {0}", PCscreenshotFile);
            }
            catch (Ivi.Driver.InstrumentStatusException e)
            {
                Console.WriteLine("Instrument reports error(s):\n{0}", e.Message);
            }

            finally
            {
                Console.WriteLine("\nPress any key to finish.");
                Console.ReadKey();
            }
        }
    }
}