﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

// installed with the RsSpecAn 3.9.0 or newer
using RohdeSchwarz.RsSpecAn;

// installed with Ivi shared components 2.42+ and Ivi.NET shared components 1.4.0+
// source: http://www.ivifoundation.org
using Ivi.Driver;
using Ivi.SpecAn;

namespace Csharp_RsScope_Driver_Example
{
    class Program
    {
        static void Main(string[] args)
        {
            RsSpecAn specan = null;
            try //separate try-catch for specan initialization prevents accessing uninitialized object
            {
                //-----------------------------------------------------------
                // Initialization:
                //-----------------------------------------------------------
                // Adjust the VISA Resource string to fit your instrument
                specan = new RsSpecAn("TCPIP::192.168.1.100::INSTR", true, true, "Simulate=False");
                specan.UtilityFunctions.OPCTimeout = 3000; //Timeout for driver wait operations, e.g. trace acquisition
            }
            catch (Ivi.Driver.IOException e)
            {
                Console.WriteLine("Error initializing the driver session:\n{0}", e.Message);
                Console.WriteLine("Press any key to finish.");
                Console.ReadKey();
                return;
            }

            try //# try block to catch any InstrumentStatusException
            {
                var idnResponse = specan.UtilityFunctions.IDQueryResponse;
                Console.WriteLine("Instrument Identification string:\n{0}", idnResponse);
                specan.Utility.Reset(); // Reset the instrument, clear the Error queue
                specan.Acquisition.SweepMode = false; // Switch OFF the continuous sweep
                specan.Display.Update = true; // Display update ON - switch OFF after debugging
                //-----------------------------------------------------------
                // Basic Settings:
                //-----------------------------------------------------------
                specan.Level.Reference = 10.0; // Setting the Reference Level
                specan.Frequency.CenterFrequency = 3E9; // Setting the center frequency
                specan.Frequency.Span = 200E6; // Setting the span
                specan.SweepCoupling.ResolutionBandwidth = 100E3; // Setting the RBW
                specan.SweepCoupling.VideoBandwidth = 300E3; // Setting the VBW
                specan.SweepCoupling.SweepPoints = 10001; // Setting the sweep points
                var opc = specan.UtilityFunctions.QueryOPC; //Using *OPC? query waits until all the instrument settings are finished
                //-----------------------------------------------------------
                // SyncPoint 'SettingsApplied' - all the settings were applied
                // -----------------------------------------------------------
                specan.UtilityFunctions.OPCTimeout = 3000; // Sweep timeout - set it higher than the instrument acquisition time
                specan.Measurement.Initiate(); // Start the sweep, wait for it to finish
                // -----------------------------------------------------------
                // SyncPoint 'AcquisitionFinished' - the results are ready
                // -----------------------------------------------------------
                // Fetching the trace - only Binary format is available in drivers
                // -----------------------------------------------------------
                ISpectrum<double> trace = null;
                trace = specan.Traces["TR1"].FetchY(trace);
                Console.WriteLine("Instrument returned {0} samples in the trace structure", trace.ValidPointCount);
                // -----------------------------------------------------------
                // Setting the marker to max and querying the X and Y
                // -----------------------------------------------------------
                specan.Marker["M1"].Search.MaximumPeak(); // Set the marker to the maximum point of the entire trace
                var markerX = specan.Marker["M1"].Position;
                var markerY = specan.Marker["M1"].Amplitude;
                Console.WriteLine("Marker Frequency {0:F3} Hz, Level {1:F2} dBm", markerX, markerY);
                // -----------------------------------------------------------
                // Making an instrument screenshot and transferring the file to the PC
                // -----------------------------------------------------------
                string instrScreenshotFile = "c:\\temp\\Dev_Screenshot.png";
                string PCscreenshotFile = "c:\\temp\\PC_Screenshot.png";
                specan.HardCopy.Destination["DE1"].LanguageFormat = HcopyDeviceLang.PNG;
                specan.HardCopy.FileName = instrScreenshotFile;
                specan.HardCopy.PrintScreen(); // Make the screenshot now
                specan.UtilityFunctions.ReadToFileFromInstrument(instrScreenshotFile, PCscreenshotFile); // Copy the file from Specan to the PC
                Console.WriteLine("Screenshot file saved to PC {0}", PCscreenshotFile);
            }
            catch (Ivi.Driver.InstrumentStatusException e)
            {
                Console.WriteLine("Instrument reports error(s):\n{0}", e.Message);
            }

            finally
            {
                Console.WriteLine("\nPress any key to finish.");
                Console.ReadKey();
            }
        }
    }
}